@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org>
+ *  Copyright (C) 2003 Marco Pesenti Gritti
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,29 +16,36 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-favicon-cache.c,v 1.20.2.2 2003/12/18 12:48:42 chpe Exp $
+ *  $Id: ephy-favicon-cache.c,v 1.31 2004/02/12 19:13:52 chpe Exp $
  */
 
-#include <libxml/tree.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <string.h>
 #include <time.h>
 #include <sys/stat.h>
 
 #include "ephy-embed-persist.h"
+#include "ephy-embed-factory.h"
 #include "ephy-file-helpers.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-node-common.h"
 #include "ephy-node.h"
 #include "ephy-debug.h"
 
-#define EPHY_FAVICON_CACHE_XML_VERSION "1.0"
+#define EPHY_FAVICON_CACHE_XML_ROOT    "ephy_favicons_cache"
+#define EPHY_FAVICON_CACHE_XML_VERSION "1.1"
 
 #define EPHY_FAVICON_CACHE_OBSOLETE_DAYS 30
 
 static void ephy_favicon_cache_class_init (EphyFaviconCacheClass *klass);
-static void ephy_favicon_cache_init (EphyFaviconCache *ma);
-static void ephy_favicon_cache_finalize (GObject *object);
+static void ephy_favicon_cache_init	  (EphyFaviconCache *ma);
+static void ephy_favicon_cache_finalize	  (GObject *object);
+
+#define EPHY_FAVICON_CACHE_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_FAVICON_CACHE, EphyFaviconCachePrivate))
 
 struct EphyFaviconCachePrivate
 {
@@ -55,16 +63,16 @@ enum
 	LAST_SIGNAL
 };
 
-static guint ephy_favicon_cache_signals[LAST_SIGNAL] = { 0 };
+static guint signals[LAST_SIGNAL] = { 0 };
 
 static GObjectClass *parent_class = NULL;
 
 GType
 ephy_favicon_cache_get_type (void)
 {
-	static GType ephy_favicon_cache_type = 0;
+	static GType type = 0;
 
-	if (ephy_favicon_cache_type == 0)
+	if (type == 0)
 	{
 		static const GTypeInfo our_info =
 		{
@@ -79,12 +87,12 @@ ephy_favicon_cache_get_type (void)
 			(GInstanceInitFunc) ephy_favicon_cache_init
 		};
 
-		ephy_favicon_cache_type = g_type_register_static (G_TYPE_OBJECT,
-								    "EphyFaviconCache",
-								     &our_info, 0);
+		type = g_type_register_static (G_TYPE_OBJECT,
+					       "EphyFaviconCache",
+					       &our_info, 0);
 	}
 
-	return ephy_favicon_cache_type;
+	return type;
 }
 
 static void
@@ -96,7 +104,7 @@ ephy_favicon_cache_class_init (EphyFaviconCacheClass *klass)
 
 	object_class->finalize = ephy_favicon_cache_finalize;
 
-	ephy_favicon_cache_signals[CHANGED] =
+	signals[CHANGED] =
 		g_signal_new ("changed",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -106,46 +114,14 @@ ephy_favicon_cache_class_init (EphyFaviconCacheClass *klass)
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_STRING);
+
+	g_type_class_add_private (object_class, sizeof(EphyFaviconCachePrivate));
 }
 
 EphyFaviconCache *
 ephy_favicon_cache_new (void)
 {
-	EphyFaviconCache *cache;
-
-	cache = EPHY_FAVICON_CACHE (g_object_new (EPHY_TYPE_FAVICON_CACHE, NULL));
-
-	g_return_val_if_fail (cache->priv != NULL, NULL);
-
-	return cache;
-}
-
-static void
-ephy_favicon_cache_load (EphyFaviconCache *eb)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root, child;
-
-	if (g_file_test (eb->priv->xml_file, G_FILE_TEST_EXISTS) == FALSE)
-		return;
-
-	doc = xmlParseFile (eb->priv->xml_file);
-	if (doc == NULL)
-	{
-		g_warning ("Failed to load favicon cache.\n");
-		return;
-	}
-
-	root = xmlDocGetRootElement (doc);
-
-	for (child = root->children; child != NULL; child = child->next)
-	{
-		EphyNode *node;
-
-		node = ephy_node_new_from_xml (eb->priv->db, child);
-	}
-
-	xmlFreeDoc (doc);
+	return EPHY_FAVICON_CACHE (g_object_new (EPHY_TYPE_FAVICON_CACHE, NULL));
 }
 
 static gboolean
@@ -157,8 +133,8 @@ icon_is_obsolete (EphyNode *node, GDate *now)
 	last_visit = ephy_node_get_property_int
 		(node, EPHY_NODE_FAVICON_PROP_LAST_USED);
 
-        g_date_clear (&date, 1);
-        g_date_set_time (&date, last_visit);
+	g_date_clear (&date, 1);
+	g_date_set_time (&date, last_visit);
 
 	return (g_date_days_between (&date, now) >=
 		EPHY_FAVICON_CACHE_OBSOLETE_DAYS);
@@ -166,8 +142,8 @@ icon_is_obsolete (EphyNode *node, GDate *now)
 
 static void
 icons_added_cb (EphyNode *node,
-	        EphyNode *child,
-	        EphyFaviconCache *eb)
+		EphyNode *child,
+		EphyFaviconCache *eb)
 {
 	g_hash_table_insert (eb->priv->icons_hash,
 			     (char *) ephy_node_get_property_string (child, EPHY_NODE_FAVICON_PROP_URL),
@@ -192,8 +168,8 @@ remove_obsolete_icons (EphyFaviconCache *eb)
 	GDate current_date;
 
 	now = time (NULL);
-        g_date_clear (&current_date, 1);
-        g_date_set_time (&current_date, time (NULL));
+	g_date_clear (&current_date, 1);
+	g_date_set_time (&current_date, time (NULL));
 
 	children = ephy_node_get_children (eb->priv->icons);
 	for (i = 0; i < children->len; i++)
@@ -205,65 +181,36 @@ remove_obsolete_icons (EphyFaviconCache *eb)
 		if (icon_is_obsolete (kid, &current_date))
 		{
 			const char *filename;
-			const char *path;
+			char *path;
 
 			filename = ephy_node_get_property_string
 				(kid, EPHY_NODE_FAVICON_PROP_FILENAME);
 			path = g_build_filename (eb->priv->directory,
 						 filename, NULL);
 			gnome_vfs_unlink (path);
+
+			g_free (path);
 			ephy_node_unref (kid);
 		}
 	}
 }
 
-static void
-ephy_favicon_cache_save (EphyFaviconCache *eb)
-{
-	xmlDocPtr doc;
-	xmlNodePtr root;
-	GPtrArray *children;
-	int i;
-
-	/* save nodes to xml */
-	xmlIndentTreeOutput = TRUE;
-	doc = xmlNewDoc ("1.0");
-
-	root = xmlNewDocNode (doc, NULL, "ephy_favicons_cache", NULL);
-	xmlSetProp (root, "version", EPHY_FAVICON_CACHE_XML_VERSION);
-	xmlDocSetRootElement (doc, root);
-
-	children = ephy_node_get_children (eb->priv->icons);
-	for (i = 0; i < children->len; i++)
-	{
-		EphyNode *kid;
-
-		kid = g_ptr_array_index (children, i);
-
-		ephy_node_save_to_xml (kid, root);
-	}
-
-	ephy_file_save_xml (eb->priv->xml_file, doc);
-
-	xmlFreeDoc (doc);
-}
-
 static void
 ephy_favicon_cache_init (EphyFaviconCache *cache)
 {
 	EphyNodeDb *db;
 
-	cache->priv = g_new0 (EphyFaviconCachePrivate, 1);
+	cache->priv = EPHY_FAVICON_CACHE_GET_PRIVATE (cache);
 
 	db = ephy_node_db_new (EPHY_NODE_DB_SITEICONS);
 	cache->priv->db = db;
 
 	cache->priv->xml_file = g_build_filename (ephy_dot_dir (),
-					          "ephy-favicon-cache.xml",
-					          NULL);
+						  "ephy-favicon-cache.xml",
+						  NULL);
 
 	cache->priv->directory = g_build_filename (ephy_dot_dir (),
-						   "favicon_cache/",
+						   "favicon_cache",
 						   NULL);
 
 	if (g_file_test (cache->priv->directory, G_FILE_TEST_IS_DIR) == FALSE)
@@ -277,7 +224,7 @@ ephy_favicon_cache_init (EphyFaviconCache *cache)
 	cache->priv->icons_hash = g_hash_table_new (g_str_hash,
 						    g_str_equal);
 	cache->priv->downloads_hash = g_hash_table_new_full (g_str_hash,
-			                                     g_str_equal,
+							     g_str_equal,
 							     g_free,
 							     NULL);
 
@@ -285,15 +232,17 @@ ephy_favicon_cache_init (EphyFaviconCache *cache)
 	cache->priv->icons = ephy_node_new_with_id (db, ICONS_NODE_ID);
 	ephy_node_ref (cache->priv->icons);
 	ephy_node_signal_connect_object (cache->priv->icons,
-				         EPHY_NODE_CHILD_ADDED,
-				         (EphyNodeCallback) icons_added_cb,
-				         G_OBJECT (cache));
+					 EPHY_NODE_CHILD_ADDED,
+					 (EphyNodeCallback) icons_added_cb,
+					 G_OBJECT (cache));
 	ephy_node_signal_connect_object (cache->priv->icons,
-				         EPHY_NODE_CHILD_REMOVED,
-				         (EphyNodeCallback) icons_removed_cb,
-				         G_OBJECT (cache));
+					 EPHY_NODE_CHILD_REMOVED,
+					 (EphyNodeCallback) icons_removed_cb,
+					 G_OBJECT (cache));
 
-	ephy_favicon_cache_load (cache);
+	ephy_node_db_load_from_file (cache->priv->db, cache->priv->xml_file,
+				     EPHY_FAVICON_CACHE_XML_ROOT,
+				     EPHY_FAVICON_CACHE_XML_VERSION);
 }
 
 static gboolean
@@ -326,20 +275,19 @@ cleanup_downloads_hash (EphyFaviconCache *cache)
 static void
 ephy_favicon_cache_finalize (GObject *object)
 {
-	EphyFaviconCache *cache;
+	EphyFaviconCache *cache = EPHY_FAVICON_CACHE (object);
 
 	LOG ("Finalize favicon cache")
 
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (EPHY_IS_FAVICON_CACHE (object));
-
-	cache = EPHY_FAVICON_CACHE (object);
-
-	g_return_if_fail (cache->priv != NULL);
-
 	cleanup_downloads_hash (cache);
 	remove_obsolete_icons (cache);
-	ephy_favicon_cache_save (cache);
+
+	ephy_node_db_write_to_xml_safe
+		(cache->priv->db, cache->priv->xml_file,
+		 EPHY_FAVICON_CACHE_XML_ROOT,
+		 EPHY_FAVICON_CACHE_XML_VERSION,
+		 NULL,
+		 cache->priv->icons, 0, NULL);
 
 	g_free (cache->priv->xml_file);
 	g_free (cache->priv->directory);
@@ -348,40 +296,43 @@ ephy_favicon_cache_finalize (GObject *object)
 
 	g_object_unref (cache->priv->db);
 
-	g_free (cache->priv);
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static char *
 favicon_name_build (const char *url)
 {
-	char *res;
-	char *slashpos;
+	char *result, *slashpos;
 
-	res = g_strdup (url);
+	result = g_filename_from_utf8 (url, -1, NULL, NULL, NULL);
 
-	while ((slashpos = strstr (res, "/")) != NULL)
+	if (result == NULL)
+	{
+		return NULL;
+	}
+
+	while ((slashpos = strstr (result, "/")) != NULL)
+	{
 		*slashpos = '_';
+	}
 
-	return res;
+	return result;
 }
 
 static void
 favicon_download_completed_cb (EphyEmbedPersist *persist,
 			       EphyFaviconCache *cache)
 {
-	char *url;
+	const char *url;
 
-	url = g_strdup ((char *) g_object_get_data (G_OBJECT (persist), "url"));
+	url = ephy_embed_persist_get_source (persist);
 	g_return_if_fail (url != NULL);
 
 	g_hash_table_remove (cache->priv->downloads_hash, url);
+
+	g_signal_emit (G_OBJECT (cache), signals[CHANGED], 0, url);
+
 	g_object_unref (persist);
-
-	g_signal_emit (G_OBJECT (cache), ephy_favicon_cache_signals[CHANGED], 0, url);
-
-	g_free (url);
 }
 
 static void
@@ -400,18 +351,16 @@ ephy_favicon_cache_download (EphyFaviconCache *cache,
 
 	dest = g_build_filename (cache->priv->directory, filename, NULL);
 
-	persist = ephy_embed_persist_new (NULL);
+	persist = EPHY_EMBED_PERSIST
+		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
 
+	ephy_embed_persist_set_dest (persist, dest);
+	ephy_embed_persist_set_flags (persist, EMBED_PERSIST_NO_VIEW);
 	ephy_embed_persist_set_max_size (persist, 100);
-	ephy_embed_persist_set_flags    (persist, EMBED_PERSIST_BYPASSCACHE);
-	ephy_embed_persist_set_source   (persist, favicon_url);
-	ephy_embed_persist_set_dest     (persist, dest);
+	ephy_embed_persist_set_source (persist, favicon_url);
 
 	g_free (dest);
 
-	g_object_set_data_full (G_OBJECT (persist), "url",
-				g_strdup (favicon_url), g_free);
-
 	g_signal_connect (G_OBJECT (persist),
 			  "completed",
 			  G_CALLBACK (favicon_download_completed_cb),
@@ -444,18 +393,22 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 		char *filename;
 
 		filename = favicon_name_build (url);
+		if (filename == NULL)
+		{
+			return NULL;
+		}
 
 		icon = ephy_node_new (cache->priv->db);
 		g_value_init (&value, G_TYPE_STRING);
 		g_value_set_string (&value, url);
 		ephy_node_set_property (icon, EPHY_NODE_FAVICON_PROP_URL,
-				        &value);
+					&value);
 		g_value_unset (&value);
 
 		g_value_init (&value, G_TYPE_STRING);
 		g_value_set_string (&value, filename);
 		ephy_node_set_property (icon, EPHY_NODE_FAVICON_PROP_FILENAME,
-				        &value);
+					&value);
 		g_value_unset (&value);
 
 		ephy_node_add_child (cache->priv->icons, icon);
@@ -468,7 +421,7 @@ ephy_favicon_cache_get (EphyFaviconCache *cache,
 	g_value_init (&value, G_TYPE_INT);
 	g_value_set_int (&value, now);
 	ephy_node_set_property (icon, EPHY_NODE_FAVICON_PROP_LAST_USED,
-			        &value);
+				&value);
 	g_value_unset (&value);
 
 	if (g_hash_table_lookup (cache->priv->downloads_hash, url) != NULL)
