@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-bookmarks-menu.c,v 1.21 2004/01/28 18:41:58 chpe Exp $
+ *  $Id: ephy-bookmarks-menu.c,v 1.22 2004/02/11 14:38:55 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -414,7 +414,10 @@ sync_topic_properties (GtkAction *action, EphyNode *bmk)
 }
 
 static void
-topic_child_changed_cb (EphyNode *node, EphyNode *child, EphyBookmarksMenu *menu)
+topic_child_changed_cb (EphyNode *node,
+			EphyNode *child,
+			guint property_id,
+			EphyBookmarksMenu *menu)
 {
 	GtkAction *action;
 	char name[64];
