@@ -15,6 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
+ *  $Id: ephy-history-window.c,v 1.66 2004/02/04 15:59:15 piers Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -27,19 +28,21 @@
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkhbox.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkactiongroup.h>
+#include <gtk/gtktoggleaction.h>
+#include <gtk/gtkcombobox.h>
+#include <gtk/gtkuimanager.h>
 #include <gdk/gdkkeysyms.h>
-#include <bonobo/bonobo-i18n.h>
+#include <glib/gi18n.h>
 #include <libgnomeui/gnome-stock-icons.h>
 #include <string.h>
+#include <time.h>
 
 #include "ephy-node-view.h"
 #include "ephy-window.h"
 #include "ephy-history-window.h"
 #include "ephy-shell.h"
 #include "ephy-dnd.h"
-#include "egg-action-group.h"
-#include "egg-toggle-action.h"
-#include "egg-menu-merge.h"
 #include "ephy-state.h"
 #include "window-commands.h"
 #include "ephy-file-helpers.h"
@@ -50,8 +53,9 @@
 #include "toolbar.h"
 #include "ephy-stock-icons.h"
 #include "ephy-search-entry.h"
-#include "session.h"
+#include "ephy-session.h"
 #include "ephy-favicon-cache.h"
+#include "eel-gconf-extensions.h"
 
 static GtkTargetEntry page_drag_types [] =
 {
@@ -74,41 +78,51 @@ static void ephy_history_window_get_property (GObject *object,
 					      GParamSpec *pspec);
 static void ephy_history_window_dispose      (GObject *object);
 
-static void cmd_open_bookmarks_in_tabs    (EggAction *action,
+static void cmd_open_bookmarks_in_tabs    (GtkAction *action,
 					   EphyHistoryWindow *editor);
-static void cmd_open_bookmarks_in_browser (EggAction *action,
+static void cmd_open_bookmarks_in_browser (GtkAction *action,
 					   EphyHistoryWindow *editor);
-static void cmd_delete                    (EggAction *action,
+static void cmd_delete                    (GtkAction *action,
                                            EphyHistoryWindow *editor);
-static void cmd_bookmark_link             (EggAction *action,
+static void cmd_bookmark_link             (GtkAction *action,
                                            EphyHistoryWindow *editor);
-static void cmd_clear			  (EggAction *action,
+static void cmd_clear			  (GtkAction *action,
 				           EphyHistoryWindow *editor);
-static void cmd_close			  (EggAction *action,
+static void cmd_close			  (GtkAction *action,
 					   EphyHistoryWindow *editor);
-static void cmd_cut			  (EggAction *action,
+static void cmd_cut			  (GtkAction *action,
 					   EphyHistoryWindow *editor);
-static void cmd_copy			  (EggAction *action,
+static void cmd_copy			  (GtkAction *action,
 					   EphyHistoryWindow *editor);
-static void cmd_paste			  (EggAction *action,
+static void cmd_paste			  (GtkAction *action,
 					   EphyHistoryWindow *editor);
-static void cmd_select_all		  (EggAction *action,
+static void cmd_select_all		  (GtkAction *action,
 					   EphyHistoryWindow *editor);
-static void cmd_help_contents		  (EggAction *action,
+static void cmd_help_contents		  (GtkAction *action,
 					   EphyHistoryWindow *editor);
 
+#define EPHY_HISTORY_WINDOW_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_HISTORY_WINDOW, EphyHistoryWindowPrivate))
+
+#define CONF_HISTORY_DATE_FILTER "/apps/epiphany/dialogs/history_date_filter"
+#define CONF_HISTORY_VIEW_DETAILS "/apps/epiphany/dialogs/history_view_details"
+
 struct EphyHistoryWindowPrivate
 {
 	EphyHistory *history;
 	GtkWidget *sites_view;
 	GtkWidget *pages_view;
 	EphyNodeFilter *pages_filter;
+	EphyNodeFilter *sites_filter;
+	GtkWidget *time_combo;
 	GtkWidget *search_entry;
-	GtkWidget *menu_dock;
+	GtkWidget *main_vbox;
 	GtkWidget *window;
-	EggMenuMerge *ui_merge;
-	EggActionGroup *action_group;
+	GtkUIManager *ui_merge;
+	GtkActionGroup *action_group;
 	GtkWidget *confirmation_dialog;
+	EphyNode *selected_site;
+	GtkTreeViewColumn *title_col;
+	GtkTreeViewColumn *address_col;
 };
 
 enum
@@ -117,94 +131,103 @@ enum
 	PROP_HISTORY
 };
 
+enum
+{
+	TIME_TODAY,
+	TIME_LAST_TWO_DAYS,
+	TIME_LAST_THREE_DAYS,
+	TIME_EVER
+};
+
+#define TIME_EVER_STRING "ever"
+#define TIME_TODAY_STRING "today"
+#define TIME_LAST_TWO_DAYS_STRING "last_two_days"
+#define TIME_LAST_THREE_DAYS_STRING "last_three_days"
+
 static GObjectClass *parent_class = NULL;
 
-static EggActionGroupEntry ephy_history_ui_entries [] = {
+static GtkActionEntry ephy_history_ui_entries [] = {
 	/* Toplevel */
-	{ "File", N_("_File"), NULL, NULL, NULL, NULL, NULL },
-	{ "Edit", N_("_Edit"), NULL, NULL, NULL, NULL, NULL },
-	{ "View", N_("_View"), NULL, NULL, NULL, NULL, NULL },
-	{ "Help", N_("_Help"), NULL, NULL, NULL, NULL, NULL },
-	{ "FakeToplevel", (""), NULL, NULL, NULL, NULL, NULL },
+	{ "File", NULL, N_("_File") },
+	{ "Edit", NULL, N_("_Edit") },
+	{ "View", NULL, N_("_View") },
+	{ "Help", NULL, N_("_Help") },
+	{ "PopupAction", NULL, "" },
 
 	/* File Menu */
-	{ "OpenInWindow", N_("_Open in New Window"), GTK_STOCK_OPEN, "<control>O",
+	{ "OpenInWindow", GTK_STOCK_OPEN, N_("_Open in New Window"), "<control>O",
 	  N_("Open the selected history link in a new window"),
-	  G_CALLBACK (cmd_open_bookmarks_in_browser), NULL },
-	{ "OpenInTab", N_("Open in New _Tab"), NULL, "<shift><control>O",
+	  G_CALLBACK (cmd_open_bookmarks_in_browser) },
+	{ "OpenInTab", NULL, N_("Open in New _Tab"), "<shift><control>O",
 	  N_("Open the selected history link in a new tab"),
-	  G_CALLBACK (cmd_open_bookmarks_in_tabs), NULL },
-	{ "Delete", N_("_Delete"), GTK_STOCK_DELETE, NULL,
+	  G_CALLBACK (cmd_open_bookmarks_in_tabs) },
+	{ "Delete", GTK_STOCK_DELETE, N_("_Delete"), NULL,
 	  N_("Delete the selected history link"),
-	  G_CALLBACK (cmd_delete), NULL },
-	{ "BookmarkLink", N_("Boo_kmark Link..."), EPHY_STOCK_BOOKMARK_PAGE, "<control>D",
+	  G_CALLBACK (cmd_delete) },
+	{ "BookmarkLink", STOCK_ADD_BOOKMARK, N_("Boo_kmark Link..."), "<control>D",
 	  N_("Bookmark the selected history link"),
-	  G_CALLBACK (cmd_bookmark_link), NULL },
-	{ "Close", N_("_Close"), GTK_STOCK_CLOSE, "<control>W",
+	  G_CALLBACK (cmd_bookmark_link) },
+	{ "Close", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
 	  N_("Close the history window"),
-	  G_CALLBACK (cmd_close), NULL },
+	  G_CALLBACK (cmd_close) },
 
 	/* Edit Menu */
-	{ "Cut", N_("Cu_t"), GTK_STOCK_CUT, "<control>X",
+	{ "Cut", GTK_STOCK_CUT, N_("Cu_t"), "<control>X",
 	  N_("Cut the selection"),
-	  G_CALLBACK (cmd_cut), NULL },
-	{ "Copy", N_("_Copy"), GTK_STOCK_COPY, "<control>C",
+	  G_CALLBACK (cmd_cut) },
+	{ "Copy", GTK_STOCK_COPY, N_("_Copy"), "<control>C",
 	  N_("Copy the selection"),
-	  G_CALLBACK (cmd_copy), NULL },
-	{ "Paste", N_("_Paste"), GTK_STOCK_PASTE, "<control>V",
+	  G_CALLBACK (cmd_copy) },
+	{ "Paste", GTK_STOCK_PASTE, N_("_Paste"), "<control>V",
 	  N_("Paste the clipboard"),
-	  G_CALLBACK (cmd_paste), NULL },
-	{ "SelectAll", N_("Select _All"), NULL, "<control>A",
+	  G_CALLBACK (cmd_paste) },
+	{ "SelectAll", NULL, N_("Select _All"), "<control>A",
 	  N_("Select all history links or text"),
-	  G_CALLBACK (cmd_select_all), NULL },
-	{ "Clear", N_("C_lear History"), GTK_STOCK_CLEAR, NULL,
+	  G_CALLBACK (cmd_select_all) },
+	{ "Clear", GTK_STOCK_CLEAR, N_("C_lear History"), NULL,
 	  N_("Clear your browsing history"),
-	  G_CALLBACK (cmd_clear), NULL },
-
-	/* View Menu */
-	{ "ViewTitle", N_("_Title"), NULL, NULL,
-	  N_("Show only the title column"),
-	  NULL, NULL, RADIO_ACTION, NULL },
-	{ "ViewLocation", N_("_Address"), NULL, NULL,
-	  N_("Show only the address column"),
-	  NULL, NULL, RADIO_ACTION, "ViewTitle" },
-	{ "ViewTitleLocation", N_("T_itle and Address"), NULL, NULL,
-	  N_("Show both the title and address columns"),
-	  NULL, NULL, RADIO_ACTION, "ViewTitle" },
+	  G_CALLBACK (cmd_clear) },
 
 	/* Help Menu */
-	{ "HelpContents", N_("_Contents"), GTK_STOCK_HELP, "F1",
+	{ "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1",
 	  N_("Display history help"),
-	  G_CALLBACK (cmd_help_contents), NULL },
-	{ "HelpAbout", N_("_About"), GNOME_STOCK_ABOUT, NULL,
+	  G_CALLBACK (cmd_help_contents) },
+	{ "HelpAbout", GNOME_STOCK_ABOUT, N_("_About"), NULL,
 	  N_("Display credits for the web browser creators"),
-	  G_CALLBACK (window_cmd_help_about), NULL },
+	  G_CALLBACK (window_cmd_help_about) },
 };
 static guint ephy_history_ui_n_entries = G_N_ELEMENTS (ephy_history_ui_entries);
 
+enum
+{
+	VIEW_TITLE,
+	VIEW_ADDRESS,
+	VIEW_TITLE_AND_ADDRESS
+};
+
+static GtkRadioActionEntry ephy_history_radio_entries [] =
+{
+	/* View Menu */
+	{ "ViewTitle", NULL, N_("_Title"), NULL,
+	  N_("Show only the title column"), VIEW_TITLE },
+	{ "ViewAddress", NULL, N_("_Address"), NULL,
+	  N_("Show only the address column"), VIEW_ADDRESS },
+	{ "ViewTitleAddress", NULL, N_("T_itle and Address"), NULL,
+	  N_("Show both the title and address columns"),
+	  VIEW_TITLE_AND_ADDRESS } 
+};
+static guint ephy_history_n_radio_entries = G_N_ELEMENTS (ephy_history_radio_entries);
+
+
 static void
 confirmation_dialog_response_cb (GtkDialog *dialog, gint response,
 				 EphyHistoryWindow *editor)
 {
-	const GList *windows;
-	Session *session;
-
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
 	if (response != GTK_RESPONSE_OK)
 		return;
 
-	session = SESSION (ephy_shell_get_session (ephy_shell));
-	windows = session_get_windows (session);
-
-	for (; windows != NULL; windows = windows->next)
-	{
-		Toolbar *t;
-
-		t = ephy_window_get_toolbar (EPHY_WINDOW (windows->data));
-		toolbar_clear_location_history (t);
-	}
-
 	ephy_history_clear (editor->priv->history);
 }
 
@@ -220,7 +243,7 @@ confirmation_dialog_construct (EphyHistoryWindow *editor)
 	GtkWidget *align;
 	char *str;
 
-	dialog = gtk_dialog_new_with_buttons (_("Clear history"),
+	dialog = gtk_dialog_new_with_buttons (_("Clear History"),
 					     GTK_WINDOW (editor),
 					     GTK_DIALOG_DESTROY_WITH_PARENT |
 					     GTK_DIALOG_NO_SEPARATOR,
@@ -256,7 +279,8 @@ confirmation_dialog_construct (EphyHistoryWindow *editor)
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
-	hbox = gtk_hbox_new (FALSE, 6);
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
 	gtk_widget_show (hbox);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), hbox,
 			    TRUE, TRUE, 0);
@@ -296,7 +320,7 @@ confirmation_dialog_construct (EphyHistoryWindow *editor)
 }
 
 static void
-cmd_clear (EggAction *action,
+cmd_clear (GtkAction *action,
 	   EphyHistoryWindow *editor)
 {
 	if (editor->priv->confirmation_dialog == NULL)
@@ -310,7 +334,7 @@ cmd_clear (EggAction *action,
 }
 
 static void
-cmd_close (EggAction *action,
+cmd_close (GtkAction *action,
 	   EphyHistoryWindow *editor)
 {
 	if (editor->priv->confirmation_dialog != NULL)
@@ -329,12 +353,15 @@ get_target_window (EphyHistoryWindow *editor)
 	}
 	else
 	{
-		return GTK_WIDGET (ephy_shell_get_active_window (ephy_shell));
+		EphySession *session;
+
+		session = EPHY_SESSION (ephy_shell_get_session (ephy_shell));
+		return GTK_WIDGET (ephy_session_get_active_window (session));
 	}
 }
 
 static void
-cmd_open_bookmarks_in_tabs (EggAction *action,
+cmd_open_bookmarks_in_tabs (GtkAction *action,
 			    EphyHistoryWindow *editor)
 {
 	EphyWindow *window;
@@ -360,7 +387,7 @@ cmd_open_bookmarks_in_tabs (EggAction *action,
 }
 
 static void
-cmd_open_bookmarks_in_browser (EggAction *action,
+cmd_open_bookmarks_in_browser (GtkAction *action,
 			       EphyHistoryWindow *editor)
 {
 	EphyWindow *window;
@@ -387,7 +414,7 @@ cmd_open_bookmarks_in_browser (EggAction *action,
 }
 
 static void
-cmd_cut (EggAction *action,
+cmd_cut (GtkAction *action,
 	 EphyHistoryWindow *editor)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (editor));
@@ -399,7 +426,7 @@ cmd_cut (EggAction *action,
 }
 
 static void
-cmd_copy (EggAction *action,
+cmd_copy (GtkAction *action,
 	  EphyHistoryWindow *editor)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (editor));
@@ -428,7 +455,7 @@ cmd_copy (EggAction *action,
 }
 
 static void
-cmd_paste (EggAction *action,
+cmd_paste (GtkAction *action,
 	   EphyHistoryWindow *editor)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (editor));
@@ -440,7 +467,7 @@ cmd_paste (EggAction *action,
 }
 
 static void
-cmd_select_all (EggAction *action,
+cmd_select_all (GtkAction *action,
 		EphyHistoryWindow *editor)
 {
 	GtkWidget *widget = gtk_window_get_focus (GTK_WINDOW (editor));
@@ -460,7 +487,7 @@ cmd_select_all (EggAction *action,
 }
 
 static void
-cmd_delete (EggAction *action,
+cmd_delete (GtkAction *action,
             EphyHistoryWindow *editor)
 {
 	if (ephy_node_view_is_target (EPHY_NODE_VIEW (editor->priv->pages_view)))
@@ -470,7 +497,7 @@ cmd_delete (EggAction *action,
 }
 
 static void
-cmd_bookmark_link (EggAction *action,
+cmd_bookmark_link (GtkAction *action,
                    EphyHistoryWindow *editor)
 {
         GtkWindow *window;
@@ -506,7 +533,7 @@ cmd_bookmark_link (EggAction *action,
 }
 
 static void
-cmd_help_contents (EggAction *action,
+cmd_help_contents (GtkAction *action,
 		   EphyHistoryWindow *editor)
 {
 	ephy_gui_help (GTK_WINDOW (editor),
@@ -514,6 +541,52 @@ cmd_help_contents (EggAction *action,
 		       "ephy-managing-history");
 }
 
+static void
+set_columns_visibility (EphyHistoryWindow *view, int value)
+{
+	switch (value)
+	{
+		case VIEW_TITLE:
+			gtk_tree_view_column_set_visible (view->priv->title_col, TRUE);
+			gtk_tree_view_column_set_visible (view->priv->address_col, FALSE);
+			break;
+		case VIEW_ADDRESS:
+			gtk_tree_view_column_set_visible (view->priv->title_col, FALSE);
+			gtk_tree_view_column_set_visible (view->priv->address_col, TRUE);
+			break;
+		case VIEW_TITLE_AND_ADDRESS:
+			gtk_tree_view_column_set_visible (view->priv->title_col, TRUE);
+			gtk_tree_view_column_set_visible (view->priv->address_col, TRUE);
+			break;
+	}
+}
+
+static void
+cmd_view_columns (GtkAction *action,
+		  GtkRadioAction *current,
+		  EphyHistoryWindow *view)
+{
+	int value;
+	GSList *svalues = NULL;
+
+	value = gtk_radio_action_get_current_value (current);
+	set_columns_visibility (view, value);
+
+	switch (value)
+	{
+		case VIEW_TITLE:
+			svalues = g_slist_append (svalues, (gpointer)"title");
+			break;
+		case VIEW_TITLE_AND_ADDRESS:
+			svalues = g_slist_append (svalues, (gpointer)"title");
+			svalues = g_slist_append (svalues, (gpointer)"address");
+			break;
+	}
+
+	eel_gconf_set_string_list (CONF_HISTORY_VIEW_DETAILS, svalues);
+	g_slist_free (svalues);
+}
+
 GType
 ephy_history_window_get_type (void)
 {
@@ -573,27 +646,21 @@ ephy_history_window_class_init (EphyHistoryWindowClass *klass)
 					 g_param_spec_object ("history",
 							      "Global history",
 							      "Global History",
-							      EPHY_HISTORY_TYPE,
+							      EPHY_TYPE_HISTORY,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_type_class_add_private (object_class, sizeof(EphyHistoryWindowPrivate));
 }
 
 static void
 ephy_history_window_finalize (GObject *object)
 {
-	EphyHistoryWindow *editor;
-
-	g_return_if_fail (object != NULL);
-	g_return_if_fail (EPHY_IS_HISTORY_WINDOW (object));
-
-	editor = EPHY_HISTORY_WINDOW (object);
-
-	g_return_if_fail (editor->priv != NULL);
+	EphyHistoryWindow *editor = EPHY_HISTORY_WINDOW (object);
 
 	g_object_unref (G_OBJECT (editor->priv->pages_filter));
+	g_object_unref (G_OBJECT (editor->priv->sites_filter));
 
 	g_object_unref (editor->priv->action_group);
-	egg_menu_merge_remove_action_group (editor->priv->ui_merge,
-					    editor->priv->action_group);
 	g_object_unref (editor->priv->ui_merge);
 
 	if (editor->priv->window)
@@ -603,8 +670,6 @@ ephy_history_window_finalize (GObject *object)
                          (gpointer *)&editor->priv->window);
 	}
 
-	g_free (editor->priv);
-
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -614,15 +679,12 @@ ephy_history_window_node_activated_cb (GtkWidget *view,
 					 EphyHistoryWindow *editor)
 {
 	const char *location;
-	EphyWindow *window;
 
 	location = ephy_node_get_property_string
 		(node, EPHY_NODE_PAGE_PROP_LOCATION);
 	g_return_if_fail (location != NULL);
 
-	window = EPHY_WINDOW (get_target_window (editor));
-	
-	ephy_shell_new_tab (ephy_shell, window, NULL, location,
+	ephy_shell_new_tab (ephy_shell, NULL, NULL, location,
 			    EPHY_NEW_TAB_OPEN_PAGE);
 }
 
@@ -634,8 +696,8 @@ ephy_history_window_update_menu (EphyHistoryWindow *editor)
 	gboolean pages_focus, pages_selection;
 	gboolean pages_multiple_selection;
 	gboolean delete, bookmark_page;
-	EggActionGroup *action_group;
-	EggAction *action;
+	GtkActionGroup *action_group;
+	GtkAction *action;
 	char *open_in_window_label, *open_in_tab_label, *copy_label;
 	GtkWidget *focus_widget;
 
@@ -693,24 +755,24 @@ ephy_history_window_update_menu (EphyHistoryWindow *editor)
 	bookmark_page = (pages_focus && pages_selection && !pages_multiple_selection);
 
 	action_group = editor->priv->action_group;
-	action = egg_action_group_get_action (action_group, "OpenInWindow");
+	action = gtk_action_group_get_action (action_group, "OpenInWindow");
 	g_object_set (action, "sensitive", open_in_window, NULL);
 	g_object_set (action, "label", open_in_window_label, NULL);
-	action = egg_action_group_get_action (action_group, "OpenInTab");
+	action = gtk_action_group_get_action (action_group, "OpenInTab");
 	g_object_set (action, "sensitive", open_in_tab, NULL);
 	g_object_set (action, "label", open_in_tab_label, NULL);
-	action = egg_action_group_get_action (action_group, "Cut");
+	action = gtk_action_group_get_action (action_group, "Cut");
 	g_object_set (action, "sensitive", cut, NULL);
-	action = egg_action_group_get_action (action_group, "Copy");
+	action = gtk_action_group_get_action (action_group, "Copy");
 	g_object_set (action, "sensitive", copy, NULL);
 	g_object_set (action, "label", copy_label, NULL);
-	action = egg_action_group_get_action (action_group, "Paste");
+	action = gtk_action_group_get_action (action_group, "Paste");
 	g_object_set (action, "sensitive", paste, NULL);
-	action = egg_action_group_get_action (action_group, "SelectAll");
+	action = gtk_action_group_get_action (action_group, "SelectAll");
 	g_object_set (action, "sensitive", select_all, NULL);
-	action = egg_action_group_get_action (action_group, "Delete");
+	action = gtk_action_group_get_action (action_group, "Delete");
 	g_object_set (action, "sensitive", delete, NULL);
-	action = egg_action_group_get_action (action_group, "BookmarkLink");
+	action = gtk_action_group_get_action (action_group, "BookmarkLink");
 	g_object_set (action, "sensitive", bookmark_page, NULL);
 }
 
@@ -770,24 +832,12 @@ ephy_history_window_show_popup_cb (GtkWidget *view,
 {
 	GtkWidget *widget;
 
-	widget = egg_menu_merge_get_widget (editor->priv->ui_merge,
-					    "/popups/EphyHistoryWindowPopup");
+	widget = gtk_ui_manager_get_widget (editor->priv->ui_merge,
+					    "/EphyHistoryWindowPopup");
 	gtk_menu_popup (GTK_MENU (widget), NULL, NULL, NULL, NULL, 2,
 			gtk_get_current_event_time ());
 }
 
-static void
-pages_filter (EphyHistoryWindow *editor,
-	      EphyNode *site)
-{
-	ephy_node_filter_empty (editor->priv->pages_filter);
-	ephy_node_filter_add_expression (editor->priv->pages_filter,
-				         ephy_node_filter_expression_new (EPHY_NODE_FILTER_EXPRESSION_HAS_PARENT,
-								          site),
-				         0);
-	ephy_node_filter_done_changing (editor->priv->pages_filter);
-}
-
 static gboolean
 key_pressed_cb (EphyNodeView *view,
 		GdkEventKey *event,
@@ -807,6 +857,114 @@ key_pressed_cb (EphyNodeView *view,
 	return FALSE;
 }
 
+static void
+add_by_site_filter (EphyHistoryWindow *editor, EphyNodeFilter *filter, int level)
+{
+	if (editor->priv->selected_site == NULL) return;
+
+	ephy_node_filter_add_expression
+		(filter, ephy_node_filter_expression_new
+				(EPHY_NODE_FILTER_EXPRESSION_HAS_PARENT,
+				 editor->priv->selected_site),
+		 level);
+}
+
+#define SEC_PER_DAY (60 * 60 * 24)
+
+static void
+add_by_date_filter (EphyHistoryWindow *editor, EphyNodeFilter *filter, int level,
+		    EphyNode *equals)
+{
+	GTime now, cmp_time, days;
+	int time_range;
+
+	/* FIXME this is probably wrong for timezones */
+
+	time_range = gtk_combo_box_get_active
+		(GTK_COMBO_BOX (editor->priv->time_combo));
+
+	now = time (NULL);
+	days = now / SEC_PER_DAY;
+
+	switch (time_range)
+	{
+		case TIME_EVER:
+			return;
+		case TIME_TODAY:
+			break;
+		case TIME_LAST_TWO_DAYS:
+			days -= 1;
+			break;
+		case TIME_LAST_THREE_DAYS:
+			days -= 2;
+			break;
+	}
+
+	cmp_time = days * SEC_PER_DAY;
+
+	ephy_node_filter_add_expression
+		(filter, ephy_node_filter_expression_new
+				(EPHY_NODE_FILTER_EXPRESSION_INT_PROP_BIGGER_THAN,
+				 EPHY_NODE_PAGE_PROP_LAST_VISIT, cmp_time),
+		 level);
+
+	if (equals == NULL) return;
+
+	ephy_node_filter_add_expression
+		(filter, ephy_node_filter_expression_new
+				(EPHY_NODE_FILTER_EXPRESSION_EQUALS, equals),
+		 0);
+}
+
+static void
+add_by_word_filter (EphyHistoryWindow *editor, EphyNodeFilter *filter, int level)
+{
+	const char *search_text;
+
+	search_text = gtk_entry_get_text (GTK_ENTRY (editor->priv->search_entry));
+	if (search_text == NULL) return;
+
+	ephy_node_filter_add_expression
+		(filter, ephy_node_filter_expression_new
+				(EPHY_NODE_FILTER_EXPRESSION_STRING_PROP_CONTAINS,
+				 EPHY_NODE_PAGE_PROP_TITLE, search_text),
+		 level);
+
+	ephy_node_filter_add_expression
+		(filter, ephy_node_filter_expression_new
+				(EPHY_NODE_FILTER_EXPRESSION_STRING_PROP_CONTAINS,
+				 EPHY_NODE_PAGE_PROP_LOCATION, search_text),
+		 level);
+}
+
+static void
+setup_filters (EphyHistoryWindow *editor,
+	       gboolean pages, gboolean sites)
+{
+	LOG ("Setup filters for pages %d and sites %d", pages, sites);
+
+	if (pages)
+	{
+		ephy_node_filter_empty (editor->priv->pages_filter);
+
+		add_by_date_filter (editor, editor->priv->pages_filter, 0, NULL);
+		add_by_word_filter (editor, editor->priv->pages_filter, 1);
+		add_by_site_filter (editor, editor->priv->pages_filter, 2);
+
+		ephy_node_filter_done_changing (editor->priv->pages_filter);
+	}
+
+	if (sites)
+	{
+		ephy_node_filter_empty (editor->priv->sites_filter);
+
+		add_by_date_filter (editor, editor->priv->sites_filter, 0,
+				    ephy_history_get_pages (editor->priv->history));
+
+		ephy_node_filter_done_changing (editor->priv->sites_filter);
+	}
+}
+
 static void
 site_node_selected_cb (EphyNodeView *view,
 		       EphyNode *node,
@@ -814,6 +972,10 @@ site_node_selected_cb (EphyNodeView *view,
 {
 	EphyNode *pages;
 
+	if (editor->priv->selected_site == node) return;
+
+	editor->priv->selected_site = node;
+
 	if (node == NULL)
 	{
 		pages = ephy_history_get_pages (editor->priv->history);
@@ -822,7 +984,7 @@ site_node_selected_cb (EphyNodeView *view,
 	else
 	{
 		ephy_search_entry_clear (EPHY_SEARCH_ENTRY (editor->priv->search_entry));
-		pages_filter (editor, node);
+		setup_filters (editor, TRUE, FALSE);
 	}
 }
 
@@ -836,6 +998,7 @@ search_entry_search_cb (GtkWidget *entry, char *search_text, EphyHistoryWindow *
 		 G_CALLBACK (site_node_selected_cb),
 		 editor);
 	all = ephy_history_get_pages (editor->priv->history);
+	editor->priv->selected_site = all;
 	ephy_node_view_select_node (EPHY_NODE_VIEW (editor->priv->sites_view),
 				    all);
 	g_signal_handlers_unblock_by_func
@@ -843,27 +1006,22 @@ search_entry_search_cb (GtkWidget *entry, char *search_text, EphyHistoryWindow *
 		 G_CALLBACK (site_node_selected_cb),
 		 editor);
 
-	ephy_node_filter_empty (editor->priv->pages_filter);
-	ephy_node_filter_add_expression (editor->priv->pages_filter,
-				         ephy_node_filter_expression_new (EPHY_NODE_FILTER_EXPRESSION_STRING_PROP_CONTAINS,
-								          EPHY_NODE_PAGE_PROP_TITLE,
-								          search_text),
-				         0);
-	ephy_node_filter_add_expression (editor->priv->pages_filter,
-				         ephy_node_filter_expression_new (EPHY_NODE_FILTER_EXPRESSION_STRING_PROP_CONTAINS,
-								          EPHY_NODE_PAGE_PROP_LOCATION,
-								          search_text),
-				         0);
-	ephy_node_filter_done_changing (editor->priv->pages_filter);
+	setup_filters (editor, TRUE, FALSE);
+}
+
+static void
+time_combo_changed_cb (GtkWidget *combo, EphyHistoryWindow *editor)
+{
+	setup_filters (editor, TRUE, TRUE);
 }
 
 static GtkWidget *
 build_search_box (EphyHistoryWindow *editor)
 {
-	GtkWidget *box;
-	GtkWidget *label;
-	GtkWidget *entry;
+	GtkWidget *box, *label, *entry;
+	GtkWidget *combo;
 	char *str;
+	int time_range;
 
 	box = gtk_hbox_new (FALSE, 6);
 	gtk_container_set_border_width (GTK_CONTAINER (box), 6);
@@ -874,9 +1032,6 @@ build_search_box (EphyHistoryWindow *editor)
 	add_entry_monitor (editor, entry);
 	editor->priv->search_entry = entry;
 	gtk_widget_show (entry);
-	g_signal_connect (G_OBJECT (entry), "search",
-			  G_CALLBACK (search_entry_search_cb),
-			  editor);
 
 	label = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -886,18 +1041,66 @@ build_search_box (EphyHistoryWindow *editor)
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
 	gtk_widget_show (label);
 
+	combo = gtk_combo_box_new_text ();
+	gtk_widget_show (combo);
+
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Today"));
+	str = g_strdup_printf (ngettext ("Last %d day", "Last %d days", 2), 2);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), str);
+	g_free (str);
+	str = g_strdup_printf (ngettext ("Last %d day", "Last %d days", 3), 3);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), str);
+	g_free (str);
+	/* keep this in sync with embed/ephy-history.c's HISTORY_PAGE_OBSOLETE_DAYS */
+	str = g_strdup_printf (ngettext ("Last %d day", "Last %d days", 10), 10);
+	gtk_combo_box_append_text (GTK_COMBO_BOX (combo), str);
+	g_free (str);
+
+	str = eel_gconf_get_string (CONF_HISTORY_DATE_FILTER);
+	if (str && strcmp (TIME_TODAY_STRING, str) == 0)
+	{
+		time_range = TIME_TODAY;
+	}
+	else if (str && strcmp (TIME_LAST_TWO_DAYS_STRING, str) == 0)
+	{
+		time_range = TIME_LAST_TWO_DAYS;
+	}
+	else if (str && strcmp (TIME_LAST_THREE_DAYS_STRING, str) == 0)
+	{
+		time_range = TIME_LAST_THREE_DAYS;
+	}
+	else
+	{
+		time_range = TIME_EVER;
+	}
+	g_free (str);
+
+	gtk_combo_box_set_active (GTK_COMBO_BOX (combo),
+				  time_range);
+
+	editor->priv->time_combo = combo;
+
 	gtk_box_pack_start (GTK_BOX (box),
 			    label, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (box),
 			    entry, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box),
+			    combo, FALSE, TRUE, 0);
+
+	g_signal_connect (combo, "changed",
+			  G_CALLBACK (time_combo_changed_cb),
+			  editor);
+	g_signal_connect (G_OBJECT (entry), "search",
+			  G_CALLBACK (search_entry_search_cb),
+			  editor);
 
 	return box;
 }
 
 static void
-add_widget (EggMenuMerge *merge, GtkWidget *widget, EphyHistoryWindow *editor)
+add_widget (GtkUIManager *merge, GtkWidget *widget, EphyHistoryWindow *editor)
 {
-	gtk_box_pack_start (GTK_BOX (editor->priv->menu_dock),
+	gtk_box_pack_start (GTK_BOX (editor->priv->main_vbox),
 			    widget, FALSE, FALSE, 0);
 	gtk_widget_show (widget);
 }
@@ -930,7 +1133,7 @@ provide_favicon (EphyNode *node, GValue *value, gpointer user_data)
 	}
 
 	g_value_init (value, GDK_TYPE_PIXBUF);
-	g_value_set_object_take_ownership (value, pixbuf);
+	g_value_take_object (value, pixbuf);
 }
 
 static void
@@ -939,6 +1142,39 @@ view_selection_changed_cb (GtkWidget *view, EphyHistoryWindow *editor)
 	ephy_history_window_update_menu (editor);
 }
 
+static int
+get_details_value (void)
+{
+	int value;
+	GSList *svalues;
+
+	svalues = eel_gconf_get_string_list (CONF_HISTORY_VIEW_DETAILS);
+	if (svalues == NULL) return VIEW_TITLE_AND_ADDRESS;
+
+	if (g_slist_find_custom (svalues, "title", (GCompareFunc)strcmp) &&
+	    g_slist_find_custom (svalues, "address", (GCompareFunc)strcmp))
+	{
+		value = VIEW_TITLE_AND_ADDRESS;
+	}
+	else if (g_slist_find_custom (svalues, "title", (GCompareFunc)strcmp))
+	{
+		value = VIEW_TITLE;
+	}
+	else if (g_slist_find_custom (svalues, "address", (GCompareFunc)strcmp))
+	{
+		value = VIEW_ADDRESS;
+	}
+	else
+	{
+		value = VIEW_TITLE_AND_ADDRESS;
+	}
+
+	g_slist_foreach (svalues, (GFunc) g_free, NULL);
+	g_slist_free (svalues);
+
+	return value;
+}
+
 static void
 ephy_history_window_construct (EphyHistoryWindow *editor)
 {
@@ -948,11 +1184,10 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	GtkWidget *pages_view, *sites_view;
 	GtkWidget *scrolled_window;
 	EphyNode *node;
-	EggMenuMerge *ui_merge;
-	EggActionGroup *action_group;
-	EggAction *action;
+	GtkUIManager *ui_merge;
+	GtkActionGroup *action_group;
 	GdkPixbuf *icon;
-	int i, col_id;
+	int col_id, details_value;
 
 	gtk_window_set_title (GTK_WINDOW (editor), _("History"));
 
@@ -965,37 +1200,40 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	g_signal_connect (editor, "delete_event",
 			  G_CALLBACK (delete_event_cb), NULL);
 
-	for (i = 0; i < ephy_history_ui_n_entries; i++)
-	{
-		ephy_history_ui_entries[i].user_data = editor;
-	}
+	editor->priv->main_vbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (editor->priv->main_vbox);
+	gtk_container_add (GTK_CONTAINER (editor), editor->priv->main_vbox);
 
-	editor->priv->menu_dock = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (editor->priv->menu_dock);
-	gtk_container_add (GTK_CONTAINER (editor), editor->priv->menu_dock);
-
-	ui_merge = egg_menu_merge_new ();
+	ui_merge = gtk_ui_manager_new ();
 	g_signal_connect (ui_merge, "add_widget", G_CALLBACK (add_widget), editor);
-	action_group = egg_action_group_new ("PopupActions");
-	egg_action_group_add_actions (action_group, ephy_history_ui_entries,
-				      ephy_history_ui_n_entries);
-	egg_menu_merge_insert_action_group (ui_merge,
+	gtk_window_add_accel_group (GTK_WINDOW (editor),
+				    gtk_ui_manager_get_accel_group (ui_merge));
+	action_group = gtk_action_group_new ("PopupActions");
+	gtk_action_group_set_translation_domain (action_group, NULL);
+	gtk_action_group_add_actions (action_group, ephy_history_ui_entries,
+				      ephy_history_ui_n_entries, editor);
+
+	details_value = get_details_value ();
+	gtk_action_group_add_radio_actions (action_group,
+					    ephy_history_radio_entries,
+					    ephy_history_n_radio_entries,
+					    details_value,
+					    G_CALLBACK (cmd_view_columns), 
+					    editor);
+
+	gtk_ui_manager_insert_action_group (ui_merge,
 					    action_group, 0);
-	egg_menu_merge_add_ui_from_file (ui_merge,
+	gtk_ui_manager_add_ui_from_file (ui_merge,
 				         ephy_file ("epiphany-history-window-ui.xml"),
 				         NULL);
-	gtk_window_add_accel_group (GTK_WINDOW (editor), ui_merge->accel_group);
-	egg_menu_merge_ensure_update (ui_merge);
+	gtk_ui_manager_ensure_update (ui_merge);
 	editor->priv->ui_merge = ui_merge;
 	editor->priv->action_group = action_group;
 
-	/* Fixme: We should implement gconf prefs for monitoring this setting */
-	action = egg_action_group_get_action (action_group, "ViewTitle");
-	egg_toggle_action_set_active (EGG_TOGGLE_ACTION (action), TRUE);
-
 	hpaned = gtk_hpaned_new ();
 	gtk_container_set_border_width (GTK_CONTAINER (hpaned), 0);
-	gtk_container_add (GTK_CONTAINER (editor->priv->menu_dock), hpaned);
+	gtk_box_pack_end (GTK_BOX (editor->priv->main_vbox), hpaned,
+			  TRUE, TRUE, 0);
 	gtk_widget_show (hpaned);
 
 	g_assert (editor->priv->history);
@@ -1011,14 +1249,13 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 					NULL);
 	gtk_paned_add1 (GTK_PANED (hpaned), scrolled_window);
 	gtk_widget_show (scrolled_window);
-	sites_view = ephy_node_view_new (node, NULL);
+	editor->priv->sites_filter = ephy_node_filter_new ();
+	sites_view = ephy_node_view_new (node, editor->priv->sites_filter);
 	add_focus_monitor (editor, sites_view);
 	col_id = ephy_node_view_add_data_column (EPHY_NODE_VIEW (sites_view),
 					         G_TYPE_STRING,
 					         EPHY_NODE_PAGE_PROP_LOCATION,
 						 NULL, NULL);
-	ephy_node_view_select_node (EPHY_NODE_VIEW (sites_view),
-				    ephy_history_get_pages (editor->priv->history));
 	ephy_node_view_enable_drag_source (EPHY_NODE_VIEW (sites_view),
 					   page_drag_types,
 				           n_page_drag_types, col_id);
@@ -1034,6 +1271,10 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	gtk_container_add (GTK_CONTAINER (scrolled_window), sites_view);
 	gtk_widget_show (sites_view);
 	editor->priv->sites_view = sites_view;
+	editor->priv->selected_site = ephy_history_get_pages (editor->priv->history);
+	ephy_node_view_select_node (EPHY_NODE_VIEW (sites_view),
+				    editor->priv->selected_site);
+
 	g_signal_connect (G_OBJECT (sites_view),
 			  "node_selected",
 			  G_CALLBACK (site_node_selected_cb),
@@ -1079,13 +1320,16 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 				         -1, EPHY_NODE_VIEW_USER_SORT |
 					 EPHY_NODE_VIEW_SEARCHABLE, NULL);
 	gtk_tree_view_column_set_max_width (col, 250);
+	editor->priv->title_col = col;
 	col = ephy_node_view_add_column (EPHY_NODE_VIEW (pages_view), _("Address"),
 				         G_TYPE_STRING, EPHY_NODE_PAGE_PROP_LOCATION,
 				         -1, EPHY_NODE_VIEW_USER_SORT, NULL);
 	gtk_tree_view_column_set_max_width (col, 200);
+	editor->priv->address_col = col;
 	gtk_container_add (GTK_CONTAINER (scrolled_window), pages_view);
 	gtk_widget_show (pages_view);
 	editor->priv->pages_view = pages_view;
+
 	g_signal_connect (G_OBJECT (pages_view),
 			  "node_activated",
 			  G_CALLBACK (ephy_history_window_node_activated_cb),
@@ -1110,6 +1354,9 @@ ephy_history_window_construct (EphyHistoryWindow *editor)
 	ephy_state_add_paned  (GTK_WIDGET (hpaned),
 			       "history_paned",
 		               130);
+
+	set_columns_visibility (editor, details_value);
+	setup_filters (editor, TRUE, TRUE);
 }
 
 void
@@ -1189,7 +1436,35 @@ ephy_history_window_get_property (GObject *object,
 static void
 ephy_history_window_init (EphyHistoryWindow *editor)
 {
-	editor->priv = g_new0 (EphyHistoryWindowPrivate, 1);
+	editor->priv = EPHY_HISTORY_WINDOW_GET_PRIVATE (editor);
+}
+
+static void
+save_date_filter (EphyHistoryWindow *editor)
+{
+	const char *time_string = NULL;
+	int time_range;
+
+	time_range = gtk_combo_box_get_active
+		(GTK_COMBO_BOX (editor->priv->time_combo));
+
+	switch (time_range)
+	{
+		case TIME_EVER:
+			time_string = TIME_EVER_STRING;
+			break;
+		case TIME_TODAY:
+			time_string = TIME_TODAY_STRING;
+			break;
+		case TIME_LAST_TWO_DAYS:
+			time_string = TIME_LAST_TWO_DAYS_STRING;
+			break;
+		case TIME_LAST_THREE_DAYS:
+			time_string = TIME_LAST_THREE_DAYS_STRING;
+			break;
+	}
+
+	eel_gconf_set_string (CONF_HISTORY_DATE_FILTER, time_string);
 }
 
 static void
@@ -1209,6 +1484,8 @@ ephy_history_window_dispose (GObject *object)
 		remove_focus_monitor (editor, editor->priv->search_entry);
 
 		editor->priv->sites_view = NULL;
+
+		save_date_filter (editor);
 	}
 
 	G_OBJECT_CLASS (parent_class)->dispose (object);
