@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: popup-commands.c,v 1.37 2004/01/23 22:50:02 marco Exp $
+ *  $Id: popup-commands.c,v 1.38 2004/02/12 19:13:54 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -374,7 +374,7 @@ popup_cmd_set_image_as_background (GtkAction *action,
 {
 	EphyEmbedEvent *info;
 	const char *location;
-	char *dest, *base;
+	char *dest, *base, *base_converted;
 	const GValue *value;
 	EphyEmbedPersist *persist;
 	EphyEmbed *embed;
@@ -390,7 +390,8 @@ popup_cmd_set_image_as_background (GtkAction *action,
 		(ephy_embed_factory_new_object ("EphyEmbedPersist"));
 
 	base = g_path_get_basename (location);
-	dest = g_build_filename (ephy_dot_dir (), base, NULL);
+	base_converted = g_filename_from_utf8 (base, -1, NULL, NULL, NULL);
+	dest = g_build_filename (ephy_dot_dir (), base_converted, NULL);
 
 	ephy_embed_persist_set_embed (persist, embed);
 	ephy_embed_persist_set_dest (persist, dest);
@@ -405,6 +406,7 @@ popup_cmd_set_image_as_background (GtkAction *action,
 
 	g_free (dest);
 	g_free (base);
+	g_free (base_converted);
 }
 
 void
